part of secure_keyboard;

const int _numericKeyRowMaxLength = 4;
const List<List<String>> _numericKeyRows = [
  const ['1', '2', '3'],
  const ['4', '5', '6'],
  const ['7', '8', '9', '0'],
  const []
];

const int _alphanumericKeyRowMaxLength = 11;
const List<List<String>> _alphanumericKeyRows = [
  const ['1', '2', '3', '4', '5', '6', '7', '8', '9', '0'],
  const ['q', 'w', 'e', 'r', 't', 'y', 'u', 'i', 'o', 'p'],
  const ['a', 's', 'd', 'f', 'g', 'h', 'j', 'k', 'l'],
  const ['z', 'x', 'c', 'v', 'b', 'n', 'm'],
  const []
];

const int _specialCharsKeyRowMaxLength = 11;
const List<List<String>> _specialCharsKeyRows = [
  const ['!', '@', '#', '\$', '%', '^', '&', '*', '(', ')'],
  const ['-', '=', '+', '{', '}', '[', ']', '\\', ':', ';'],
  const ['\"', '\'', '<', '>', ',', '.', '/', '?', '|'],
  const ['~', '`', '_', '\₩', '#', '@', '!'],
  const []
];

List<List<SecureKeyboardKey>> _getNumericKeyRows() {
  final random = Random();
  int randomIndex;
  int emptyLength;

  return List.generate(_numericKeyRows.length, (int rowNum) {
    List<SecureKeyboardKey> rowKeys = [];

    switch (rowNum) {
      case 3:
        // Clear
        rowKeys.add(_clearActionKey());

        // Backspace
        rowKeys.add(_backspaceActionKey());

        // Confirm
        rowKeys.add(_confirmActionKey());
        break;
      default:
        rowKeys = _getStringKeyRow(_numericKeyRows, rowNum);
        emptyLength = _numericKeyRowMaxLength - _numericKeyRows[rowNum].length;

        for (var i=0; i<emptyLength; i++) {
          randomIndex = random.nextInt(_numericKeyRowMaxLength);

          if (randomIndex == _numericKeyRowMaxLength - 1)
            rowKeys.add(_blankActionKey());
          else
            rowKeys.insert(randomIndex, _blankActionKey());
        }

        rowKeys.shuffle();
    }

    return rowKeys;
  });
}

List<List<SecureKeyboardKey>> _getAlphanumericKeyRows() {
  final random = Random();
  int randomIndex;
  int emptyLength;

  return List.generate(_alphanumericKeyRows.length, (int rowNum) {
    List<SecureKeyboardKey> rowKeys = [];

    switch (rowNum) {
      case 3:
        // Shift
        rowKeys.add(_shiftActionKey());

        rowKeys.addAll(_getStringKeyRow(_alphanumericKeyRows, rowNum));

        emptyLength = _alphanumericKeyRows[rowNum].length;
        randomIndex = random.nextInt(emptyLength) + 1;
        rowKeys.insert(randomIndex, _blankActionKey());

        // Backspace
        rowKeys.add(_backspaceActionKey());
        break;
      case 4:
        // SpecialChars
        rowKeys.add(_specialCharsActionKey());

        // Clear
        rowKeys.add(_clearActionKey());

        // Confirm
        rowKeys.add(_confirmActionKey());
        break;
      default:
        rowKeys = _getStringKeyRow(_alphanumericKeyRows, rowNum);
        emptyLength = _alphanumericKeyRowMaxLength - _alphanumericKeyRows[rowNum].length;

        for (var i=0; i<emptyLength; i++) {
          randomIndex = random.nextInt(_alphanumericKeyRowMaxLength);

          if (randomIndex == _alphanumericKeyRowMaxLength - 1)
            rowKeys.add(_blankActionKey());
          else
            rowKeys.insert(randomIndex, _blankActionKey());
        }

        if (rowNum == 0)
          rowKeys.shuffle();
    }

    return rowKeys;
  });
}

List<List<SecureKeyboardKey>> _getSpecialCharsKeyRows() {
  final random = Random();
  int randomIndex;
  int emptyLength;

  return List.generate(_specialCharsKeyRows.length, (int rowNum) {
    List<SecureKeyboardKey> rowKeys = [];

    switch (rowNum) {
      case 3:
        // Shift
        rowKeys.add(_shiftActionKey());

        rowKeys.addAll(_getStringKeyRow(_specialCharsKeyRows, rowNum));

        emptyLength = _specialCharsKeyRows[rowNum].length;
        randomIndex = random.nextInt(emptyLength) + 1;
        rowKeys.insert(randomIndex, _blankActionKey());

        // Backspace
        rowKeys.add(_backspaceActionKey());
        break;
      case 4:
        // SpecialChars
        rowKeys.add(_specialCharsActionKey());

        // Clear
        rowKeys.add(_clearActionKey());

        // Confirm
        rowKeys.add(_confirmActionKey());
        break;
      default:
        rowKeys = _getStringKeyRow(_specialCharsKeyRows, rowNum);
        emptyLength = _specialCharsKeyRowMaxLength - _specialCharsKeyRows[rowNum].length;

        for (var i=0; i<emptyLength; i++) {
          randomIndex = random.nextInt(_specialCharsKeyRowMaxLength);

          if (randomIndex == _specialCharsKeyRowMaxLength - 1)
            rowKeys.add(_blankActionKey());
          else
            rowKeys.insert(randomIndex, _blankActionKey());
        }
    }

    return rowKeys;
  });
}

List<SecureKeyboardKey> _getStringKeyRow(List<List<String>> keyRows, int rowNum) {
  return List.generate(keyRows[rowNum].length, (int keyNum) {
    String key = keyRows[rowNum][keyNum];

    return SecureKeyboardKey(
      text: key,
      capsText: key.toUpperCase(),
      type: SecureKeyboardKeyType.String
    );
  });
}

SecureKeyboardKey _backspaceActionKey() {
  return SecureKeyboardKey(
    type: SecureKeyboardKeyType.Action,
    action: SecureKeyboardKeyAction.Backspace
  );
}

SecureKeyboardKey _confirmActionKey() {
  return SecureKeyboardKey(
      type: SecureKeyboardKeyType.Action,
      action: SecureKeyboardKeyAction.Confirm
  );
}

SecureKeyboardKey _clearActionKey() {
  return SecureKeyboardKey(
    type: SecureKeyboardKeyType.Action,
    action: SecureKeyboardKeyAction.Clear
  );
}

SecureKeyboardKey _shiftActionKey() {
  return SecureKeyboardKey(
    type: SecureKeyboardKeyType.Action,
    action: SecureKeyboardKeyAction.Shift
  );
}

SecureKeyboardKey _blankActionKey() {
  return SecureKeyboardKey(
    type: SecureKeyboardKeyType.Action,
    action: SecureKeyboardKeyAction.Blank
  );
}

SecureKeyboardKey _specialCharsActionKey() {
  return SecureKeyboardKey(
    type: SecureKeyboardKeyType.Action,
    action: SecureKeyboardKeyAction.SpecialChars
  );
}